# -*- coding: utf-8 -*-

"""
PloneOOoTransforms

(C) 2005, 2006, ZINDEP and Contributors
http://www.zindep.com

Contact: plone@zindep.com

License: see LICENSE.txt

"""
import re, os, tempfile
from os.path import join

import uno
import unohelper
from com.sun.star.beans import PropertyValue
from Products.PloneOOoTransforms import uno_utils

from Products.PortalTransforms.interfaces import itransform

class uno_to_xx:
    """abstract class for uno to pdf command based transform
    """
    __implements__ = itransform
     
    def __init__(self, name=None, filter_name=None, out_ext=None, **kwargs):
        """ Initialization: create tmp work directory and copy the
        document into a file"""
        if name is not None:
            self.__name__ = name
        if filter_name is not None:
            self.filter_name = filter_name
        if out_ext is not None:
            self.out_ext = out_ext
            
    def name(self):
        return self.__name__

    
    def convert(self, data, cache, **kwargs):
        """Convert the document"""

        filename = kwargs.get('filename', 'unknown')
        outputfilename = filename + self.out_ext        

        tmpdir, docfullname, doc = uno_utils.loadDocument(data)
        outputfullname = docfullname + self.out_ext       

        filterName = self.filter_name
        storeProps = (
                       PropertyValue('FilterName', 0, filterName, 0),
                      )
        if kwargs.get('Author', None):
            storeProps += (
                       PropertyValue( "Author" , 0 , kwargs.get('Author'), 0 ),
                          )
        if kwargs.get('Comment', None):
            storeProps += (
                       PropertyValue( "Comment" , 0 , kwargs.get('Comment'), 0 ),
                          )
        if kwargs.get('DocumentTitle', None):
            storeProps += (
                       PropertyValue( "DocumentTitle" , 0 , kwargs.get('DocumentTitle'), 0 ),
                          )

        # pre-create a empty file for security reason
        url = unohelper.systemPathToFileUrl(outputfullname)
        doc.storeToURL(url, storeProps)

        try:
            doc.close(True)
        except com.sun.star.util.CloseVetoException:
            pass

        #read file
        f = open(outputfullname, 'r')
        out = f.read()
        f.close()

        objects = {}
        for f in os.listdir(tmpdir):
            objects[f] = open(join(tmpdir, f), 'rb').read()
                    
        uno_utils.cleanDir(tmpdir)

        cache.setData(out)
        cache.setSubObjects(objects)
        
        return cache
