# -*- coding: utf-8 -*-

"""
PloneOOoTransforms

(C) 2005, 2006, ZINDEP and Contributors
http://www.zindep.com

Contact: plone@zindep.com

License: see LICENSE.txt

"""

import re, os, tempfile
import uno
import unohelper
from com.sun.star.beans import PropertyValue
from Products.PloneOOoTransforms import uno_utils

from Products.PortalTransforms.libtransforms.commandtransform import commandtransform
from Products.CMFCore.utils import getToolByName
from DocumentTemplate.cDocumentTemplate import safe_callable


from zLOG import LOG as log
from zLOG import INFO, DEBUG, WARNING

from Products.PloneOOoTransforms.config import PROJECTNAME


class document(commandtransform):

    def __init__(self, name, data):
        """ Initialization: create tmp work directory and copy the
        document into a file"""

        commandtransform.__init__(self, name)
        name = self.name()
        self.tmpdir, self.fullname = uno_utils.initialize_tmpdir(data, filename=name)
        self.outputfile = self.fullname + '.odt'

    def convert(self,**kwargs):
        """Convert the document"""

        # used_tmpdirs :
        # we use it to clean all tpmdirs in the end of this convert function.
        used_tmpdirs = []
        

        ###########################
        # get ODT Model Document
        context = kwargs.get('context')
        model = kwargs.get('model')
        if model:
            tmpdir, tmpfilename, modeldoc = model.getUnoModel()
        else:
            tmpdir, tmpfilename, modeldoc = uno_utils.loadDocument(' ')
        used_tmpdirs.append(tmpdir)
        

        ########################
        # parameters for encode/decode
        try:
            prop = getToolByName(context, 'portal_properties')
            oootool = getToolByName(context, 'portal_oootransformtool')
            decoding = prop.site_properties.getProperty('default_charset', 'utf-8')
            encoding = oootool.getOoo_inputencode()
        except:
            pass


        ###########################
        # get Cursor
        text = modeldoc.getText()        
        cursor = text.createTextCursor()
        
        
        #############################
        # insert data default in document in flow : default range of cursor
        # if insert_inflowdoc is to True
        if not model or model.getInsert_inflowdoc():
            # load the doc to insert
            url = unohelper.systemPathToFileUrl(self.fullname)
            cursor.insertDocumentFromURL(url, () )


        if model:
            ##############################
            # DocumentInfo Fields
            for info_field in model.getInfodoc_mapfields():
                try:
                    attr = getattr(context, info_field['field'])
                    if safe_callable(attr):
                        attr = attr()
                    value = str(attr)
                    setattr(modeldoc.DocumentInfo, info_field['infodoc'], value)
                except Exception, e:
                    log("Raised error %s for uno infodoc field %s" % (e, info_field['infodoc']),
                        WARNING,
                        'PloneOOoTransforms')   


            ##############################
            # User Fields
            for user_field in model.getUser_mapfields():
                try:
                    attr = getattr(context, user_field['field'])
                    if safe_callable(attr):
                        attr = attr()
                    value = str(attr)
                    oofield = modeldoc.TextFieldMasters.getByName(user_field['user'])
                    oofield.setPropertyValue("Content", value)
                except Exception, e:
                    log("Raised error %s for uno user field %s" % (e, user_field['user']),
                        WARNING,
                        'PloneOOoTransforms')
                        
                

            ##############################
            # Placeholder Fields        
            for placeholder_field in model.getPlaceholder_mapfields():
                # get placeholder with name placeholder_field['placeholder']
                # maybe here there is better to do ...
                enum = modeldoc.getTextFields().createEnumeration()
                while enum.hasMoreElements():
                    oofield = enum.nextElement()
                    if oofield.supportsService("com.sun.star.text.TextField.JumpEdit") and oofield.getPropertyValue("PlaceHolder")==placeholder_field['placeholder']:
                        try:
                            cursor.gotoRange(oofield.getAnchor(), 0)
                            attr = getattr(context, placeholder_field['field'])
                            if safe_callable(attr):
                                attr = attr()
                            data = uno_utils.formatValue(attr)
                            #print data

                            # try to decode/encode data ...
                            try:
                                data = data.decode(decoding).encode(encoding)
                            except Exception, e:
                                log("Raised error in trying decoding/encoding %s for uno placeholder field %s" % (e, placeholder_field['placeholder']),
                                    WARNING,
                                    'PloneOOoTransforms')


                            oofieldtmpdir, oofieldfullname = self.initialize_tmpdir(data, filename = '%s.html' % placeholder_field['field'])
                            used_tmpdirs.append(oofieldtmpdir)
                            #print oofieldfullname
                            
                            # get images and fix urls 
                            images = []
                            data = uno_utils.fixUrlsImages(context, images, data)
                            filedest = open(oofieldfullname, "wb")
                            filedest.write(data)
                            filedest.close()
                            
                            for img in images:
                                pathimg = os.path.join(oofieldtmpdir, img.getId())
                                filedest = open(pathimg , "wb")
                                filedest.write(str(img.getImage().data))
                                filedest.close()
                            
                            oofieldurl = unohelper.systemPathToFileUrl(oofieldfullname)
                            cursor.insertDocumentFromURL(oofieldurl, () )
                        except Exception, e:
                            log("Raised error %s for uno placeholder field %s" % (e, placeholder_field['placeholder']),
                                WARNING,
                                'PloneOOoTransforms')

        # try to embed links graphics.
        uno_utils.imgLinkToEmbedded(modeldoc)
                    

        #######################################"
        storeProps = (
            PropertyValue( "Hidden" , 0 , True, 0 ),
            )
            
        # pre-create a empty file for security reason
        url = unohelper.systemPathToFileUrl(self.outputfile)
        modeldoc.storeToURL(url, storeProps)


        ##############################
        # Close document(s) if possible
        try:
            modeldoc.close(True)
        except com.sun.star.util.CloseVetoException:
            pass

        # try to remove all used tmpdirs
        for d in used_tmpdirs:
            try:
                uno_utils.cleanDir(d)
            except:
                pass

        
        # maigic to release some resource
        #ctx.ServiceManager


    def read(self):
        """
        return data of file
        """
        f = open(self.outputfile, 'r')
        out = f.read()
        f.close()
        return out
