import uno
import unohelper
from com.sun.star.beans import PropertyValue

import tempfile
import re
import os
import urllib, urlparse

from os.path import join, basename
from AccessControl import ModuleSecurityInfo
import shutil

from Acquisition import aq_base

from types import ListType, TupleType, ClassType, FileType, DictType, IntType
from types import StringType, UnicodeType, StringTypes

from Products.CMFCore.utils import getToolByName


security = ModuleSecurityInfo( 'Products.PloneOOoTransforms.uno_utils' )

security.declarePublic('getDesktop')
def getDesktop():
    """
    Initialize UNO ans return desktop
    """
    localContext = uno.getComponentContext()
    resolver = localContext.ServiceManager.createInstanceWithContext(
        'com.sun.star.bridge.UnoUrlResolver', localContext )
    ctx = resolver.resolve(
        'uno:socket,host=localhost,port=2002;'
        'urp;StarOffice.ComponentContext')
    smgr = ctx.ServiceManager
    desktop = smgr.createInstanceWithContext(
        'com.sun.star.frame.Desktop', ctx)
    return desktop

security.declarePublic('loadDocument')
def loadDocument(data, **kwargs):
    """
    """
    desktop = getDesktop()
    doctmpdir, docfullname = initialize_tmpdir(data, **kwargs)
    docurl = unohelper.systemPathToFileUrl(docfullname)
    # Hidden readProps disabled : see imgLinkToEmbedded to know why ...
    #readProps = ( PropertyValue( "Hidden" , 0 , True, 0 ),)
    readProps = ()
    doc = desktop.loadComponentFromURL( docurl,"_blank", 0, readProps)
    return (doctmpdir, docfullname, doc)

security.declarePublic('initialize_tmpdir')
def initialize_tmpdir(data, **kwargs):
    """
    create a temporary directory, copy input in a file there
    return the path of the tmp dir and of the input file
    """
    tmpdir = tempfile.mktemp()
    os.mkdir(tmpdir)
    filename = kwargs.get("filename", 'tmpfile')
    fullname = join(tmpdir, basename(filename))
    filedest = open(fullname , "wb").write(str(data))
    return tmpdir, fullname


security.declarePublic('cleanDir')
def cleanDir(tmpdir):
        shutil.rmtree(tmpdir)


security.declarePublic('formatValue')
def formatValue(value):
    """
    format value for inserting in ooo document
    TODO : make better formatted ;)
    """
    if type(value) in [StringType, UnicodeType, StringTypes]:
        return '<body>%s</body>' % value
    if type(value) in [ListType, TupleType]:
        if value[:1] in [StringType, UnicodeType, StringTypes]:
            return '<body>%s</body>' % ','.join(value)
        else:
            return '<body>%s</body>' % value
    if type(value) in [DictType,]:
        return '<body>%s</body>' % value

    # for files :
    if hasattr(aq_base(value), 'data'):
        return value.data

    return value


security.declarePublic('fixUrlsImages')
def fixUrlsImages(context, images, data):
    """
    fix urls of data and return it : absolute urls become relative
    in the same time, put images of data in images
    """
    URL_PATTERN = re.compile("""(?P<tag><(?:a|img) [^>]*(?:src|href)=)(?P<url>['"][^'"]*?['"])(?P<end>.*?>)""",  re.MULTILINE| re.DOTALL)


    # Replace relative urls with absolute urls
    def replaceUrls(match):
        tag = match.group('tag')
        url = match.group('url')[1:-1]      #drop the quotes
        end = match.group('end')
        # portal = getToolByName(self.context, 'portal_url').getPortalObject()

        if url.startswith('http://'):
            # same domain
            if urlparse.urlsplit(url)[1] ==  urlparse.urlsplit(context.absolute_url())[1]:
                url = urlparse.urlsplit(url)[2]
        # link put by kupu
        url_list = url.split('/')
        if  'resolveuid' in url_list:
            uuid = url_list[url_list.index('resolveuid')+1]
            reference_tool = getToolByName(context, 'reference_catalog')
            obj = reference_tool.lookupObject(uuid)
        else:
            url_abs = urlparse.urljoin(context.absolute_url()+'/',url)
            url_abs = urlparse.urlsplit(url_abs)[2]
            obj = context.restrictedTraverse(url_abs,None)

        if tag.lower().startswith('<a'):
            if obj:
                url = '"%s"' % obj.absolute_url()
                return tag + url + end
            
        if tag.lower().startswith('<img'):
            if obj:
                #we have an image to copy
                url = '"%s"' % obj.getId()                    
                images.append(obj)
            else:
                # we have a remote image to copy
                class dummyImage:
                    def __init__(self, id, data):
                        self.id = id
                        self.data = data
                    def getImage(self,):
                        return self
                    def getId(self,):
                        return self.id

                # import pdb; pdb.set_trace()
                fname = url.split('/')[-1]
                try:
                    sock = urllib.urlopen(url)
                except IOError:
                    return
                fdata  = sock.read()
                sock.close()
                images.append(dummyImage(fname, fdata))
                url = '"%s"' % fname

            return tag + url + end
                
 
        return match.group(0)

    return URL_PATTERN.sub(replaceUrls, data)


security.declarePublic('imgLinkToEmbedded')
def imgLinkToEmbedded(doc):
    """
    try to embed links graphics
    some OOo hacks ...
    """
    if len(doc.getGraphicObjects().ElementNames) > 0:
        for graphicName in doc.getGraphicObjects().ElementNames:
            oGraph = doc.getGraphicObjects().getByName(graphicName)
            if not oGraph.GraphicURL.startswith('vnd.sun') and not oGraph.GraphicURL.startswith('http'):

                # graphic is a link (and is not a remote link)
                # so we try to embed it
                # in case of remote link, OOo try to get image and if he can't,
                # he doesn't return anything ... 
                
                url = oGraph.GraphicURL
                oAnchor = oGraph.getAnchor()
            
                    
                # here a little hack to have good size :
                # viewcursor goes to range of Anchor and size of image becomes ok !
                # it's why Hidden readProps is disabled 
                myViewCursor = doc.getCurrentController().ViewCursor
                myViewCursor.gotoRange(oAnchor, False)
                
                oShape1 = doc.createInstance('com.sun.star.drawing.GraphicObjectShape')
                oShape1.GraphicURL = url
                oAnchor.getText().insertTextContent(oAnchor, oShape1, False)
                
                oShape2 = doc.createInstance('com.sun.star.drawing.GraphicObjectShape')
                oShape2.GraphicObjectFillBitmap = oShape1.GraphicObjectFillBitmap
                
                oAnchor.getText().insertTextContent(oAnchor, oShape2, False)
                oAnchor.getText().removeTextContent(oShape1)
                
                oGraph.GraphicURL = oShape2.GraphicURL
                oAnchor.getText().removeTextContent(oShape2)





